#!/bin/ksh
#
# Copyright 2003 Sun Microsystems, Inc. All Rights Reserved.
#
# @(#)ckval_lib.ksh	1.4 03/06/23
#
# ckval_lib.ksh -- Configvar "checkvalue" shell function library
#
# This library is primarily used by configvar "checkvalue" scriptlets
# to validate configuration parameters entered by the user.  It is
# expected to be imported prior to the prompting and validation of a
# set of configvars.
#
# Most functions in this library are used for data validation, but some
# exist as "helper" functions for various tasks.
#
# All public functions defined herein have a "ckv_" name prefix to
# avoid namespace collisions.  All private functions have a name prefix
# of '_ckv_"; these functions are not expected to be called from outside
# this library.
#
# Unless otherwise noted, all functions return either 0 for success or 1 for
# failure.
#
# IMPORTANT: none of these functions performs API-level parameter validation;
# that is, calling parameters are expected to be accurate as required.
#
# This library sets the "CKVAL_LIB_IS_LOADED" global variable to "true"
# to allow a client script to test whether this library is already loaded.
#
# Functions are defined in alphbetical order, with public functions defined
# at the top, and private functions defined at the bottom.
#
###

#------------------------------------------------------------------------------
# ckv_Boolean string
#
# Determine whether the string represents a valid ESM boolean value,
# and emit an error/help message if it doesn't.
#
ckv_Boolean ()
{
    el_checkboolean "$1" >/dev/null 2>&1 && return 0
    print "Please enter either 'yes' or 'no', or simply 'y' or 'n'."
    return 1
}


#------------------------------------------------------------------------------
# ckv_CharSet string charsetspec
#
# Determine whether the string is comprised solely from the character set spec.
# The charsetspec follows the form specified by tr(1) and regex(5); see those
# man pages for detailed info.  The string must be non-null.
#
# Algorithm: remove all chars specified in charsetspec; if no chars remain,
# then success is returned.
#
ckv_CharSet ()
{
    [ -n "$1" -a -z "`print -n \"$1\" | /usr/bin/tr -d \"$2\"`" ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_Clean string
#
# Output the string with leading and trailing spaces removed.  Always returns 0.
#
# NOTE: this is a cheap, not-so-great algorithm insofar that embedded spaces
# are squeezed to a single space.  Could be a problem for cases such as passwords
# or other values that allow multiple contiguous spaces.
#
ckv_Clean ()
{
    /usr/bin/expr "$*" 2>/dev/null
    return 0
}


#------------------------------------------------------------------------------
# ckv_ConfirmSuspectValue prompt
#
# Prompt the user to accept or decline a potentially invalid value.
# The default response is ALWAYS 'no'.
#
# Returns 0 if the user accepts the value, or 1 if not.
#
ckv_ConfirmSuspectValue ()
{
    el_isEsmTrue `/usr/bin/ckyorn -Q -d no -p "$1"`
}


#------------------------------------------------------------------------------
# ckv_HostExists hostname-or-ipaddress
#
# Determine if a hostname or IP address represents an existing host.
#
# Only a simple "ping" is used to validate the parameter.  The output from ping
# (either error or info) is returned in the special _ckv_HostExists_results
# variable that the caller can use to provide detailed messages.
#
ckv_HostExists ()
{
    if ckv_NonBlank "$1" ; then
	if _ckv_HostExists_results=`/usr/sbin/ping "$1" 2>&1` ; then
	    return 0
	fi
    else
	_ckv_HostExists_results='no hostname or IP address specified'
    fi

    return 1
}


#------------------------------------------------------------------------------
# ckv_LengthRange string min max
#
# Determine if the length of the string is between the pair of bounds,
# inclusively.
#
ckv_LengthRange ()
{
    ckv_MinLength "$1" $2 && ckv_MaxLength "$1" $3 >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_MaxLength string maxlength
#
# Determine if a string does not exceed a maximum length.
#
ckv_MaxLength ()
{
    [ `/usr/bin/expr "$1" : '.*'` -le $2 ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_MinLength string minlength
#
# Determine if a string is at least a minimum length.
#
ckv_MinLength ()
{
    [ `/usr/bin/expr "$1" : '.*'` -ge $2 ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_NonBlank string
#
# Determine if the string has one or more non-blank characters.  Useful for
# determining if the user entered only blank characters for a response.
#
# See also: ckv_NonNull
#
ckv_NonBlank ()
{
    [ -n "$1" -a -n `print -n "$1" | /usr/xpg4/bin/tr -d '[:blank:]'` ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_NonNull string
#
# Determine if the string has a non-zero length.  Note that the string can be
# of any set of characters, including blanks.  Useful for quickly determining
# whether the user simply pressed RETURN when prompted for a value.
#
# See also: ckv_NonBlank
#
ckv_NonNull ()
{
    [ -n "$1" ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_OracleIdentifier string
#
# Determine whether the specified string is a valid Oracle identifier.
#
ckv_OracleIdentifier ()
{
    ckv_CharSet "$1" '[:alnum:]_-$' && ckv_LengthRange "$1" 1 30
}


#------------------------------------------------------------------------------
# ckv_OracleAccountName string
#
# Determine whether the specified string is valid for an Oracle account name.
#
ckv_OracleAccountName ()
{
    if ckv_OracleIdentifier "$1" ; then
	return 0
    fi

    print "Invalid Oracle account name.  Account names are 1 to 30 characters," >&2
    print "comprising letters, digits, hyphens, dollar signs or underscores." >&2

    return 1
}


#------------------------------------------------------------------------------
# ckv_PortAssignedTo service-name port-number [protocol]
#
# Determine whether a service name is assigned to the specified port number
# and protocol, as defined in the /etc/services database.
#
# If "protocol" is not specified, then "tcp" is assumed.
#
ckv_PortAssignedTo ()
{
    __proto="$3" ; [ -z "$__proto" ] && __proto='tcp'
    __goal="$2/$__proto"

    __xx=`/bin/grep -w "$__goal" /etc/services | /usr/bin/egrep "^[^#]*$__goal" | /usr/bin/awk '{ print $1 }'`

    [ "$__xx" = "$1" ]
}


#------------------------------------------------------------------------------
# ckv_PortAvailable port-number [protocol]
#
# Determine whether the specified port is available (ie, not reserved or in use).
# The "protocol" is either "udp" or the default value of "tcp".
#
# Return values:
#   0 - The port is not in use
#   1 - The port is registered in /etc/services
#   2 - The port is currently in use in the host connection table
#   3 - The port is currently used by an ESM configvar
#   4 - The port is currently used by an the current configvar
#
ckv_PortAvailable ()
{
    __proto="$2" ; [ -z "$__proto" ] && __proto='tcp'
    __goal="$1/$__proto"

    # Check to see if is defined in another ESM configvar.  This code 
    # assumes that all configvars containing port numbers have the the 
    # word "port" (case is ignored) in the configvar name.
    #
    for _lcvn in `/bin/ls $ESM_CONFIG/configvars | /bin/grep -i port`
    do
	if [ "$_lcvn" = "$_cvn" ]; then
	    _lcvv=`el_configvar_get $_lcvn`
	    if [ "$_lcvv" = "$1" ]; then return 4 ; fi
	else
	    _lcvv=`el_configvar_get $_lcvn`
	    if [ "$_lcvv" = "$1" ]; then return 3 ; fi
	fi
    done

    # First check to see if the port is registered in /etc/services
    #
    if /bin/grep -w "$__goal" /etc/services | /usr/bin/egrep -s "^[^#]*$__goal"
    then
	return 1
    fi

    # It's not registered in /etc/services, so check the current host
    # connections table to see if some non-registered use exists.
    #
    for __qq in `/usr/bin/netstat -P tcp -f inet | /usr/bin/cut -f1 -d' ' | /usr/bin/cut -f2 -d.`
    do
	if [ "$__qq" = "$1" ] ; then return 2 ; fi
    done

    return 0    # If we get here, then the port is not in use
}


#------------------------------------------------------------------------------
# ckv_PositiveInteger string
#
# Determine if the string represents a single positive (non-negative, non-zero)
# integer.  Leading/trailing spaces are removed prior to testing.
#
ckv_PositiveInteger ()
{
    __xx=`/usr/bin/expr $1` >/dev/null 2>&1
    [ 0 -lt "__xx" ] >/dev/null 2>&1
}


#------------------------------------------------------------------------------
# ckv_PositiveIntegerRange string min max
#
# Determine if the string represents a single positive (non-negative, non-zero)
# integer that is between the pair of bounds, inclusively.  Leading/trailing
# spaces are removed prior to testing.
#
ckv_PositiveIntegerRange ()
{
    ckv_PositiveInteger "$1" && [ $1 -ge $2 -a $1 -le $3 ] >/dev/null 2>&1

}


#------------------------------------------------------------------------------
# ckv_ServiceName string
#
# Determine whether the specified string is valid for an Oracle service name.
#
ckv_ServiceName ()
{
    if ckv_OracleIdentifier "$1" ; then
	return 0
    fi

    print "Invalid service name.  Service names are 1 to 30 characters," >&2
    print "comprising letters, digits, hyphens, dollar signs or underscores." >&2

    return 1
}


#------------------------------------------------------------------------------
# ckv_SID string
#
# Determine whether the specified string is valid for an Oracle SID.
#
ckv_SID ()
{
    if ckv_OracleIdentifier "$1" ; then
	return 0
    fi

    print "Invalid Oracle SID.  SID names are 1 to 30 characters," >&2
    print "comprising letters, digits, hyphens, dollar signs or underscores." >&2

    return 1
}


#------------------------------------------------------------------------------
# ckv_TablespaceName string
#
# Determine whether the specified string is valid for an Oracle tablespace name.
#
ckv_TablespaceName ()
{
    if ckv_OracleIdentifier "$1" ; then
	return 0
    fi

    print "Invalid tablespace name.  Tablespace names are 1 to 30 characters," >&2
    print "comprising letters, digits, hyphens, dollar signs or underscores." >&2

    return 1
}


#------------------------------------------------------------------------------
# ckv_UnixAcctExists account-name
#
# Determine if the specified account exists on the local host.
#
# A check is first done to see if the target account is the same as the
# current process account.
#
# WARNING: if the caller is not the super-user, then the process will hang
# waiting for the user to enter the account password (unless the target and
# the current accounts are the same).
#
ckv_UnixAcctExists ()
{
    if ckv_NonBlank "$1" ; then
	if [ "$1" = `/usr/bin/id | /usr/bin/sed -e 's|.*(\(.*\)).*(.*).*|\1|'` ] ||
	   ( /usr/bin/su "$1" -c /usr/bin/id ) >/dev/null 2>&1
        then
	    return 0
	fi
    fi

    return 1
}


#------------------------------------------------------------------------------
# ckv_xxx string
#
# template
#
ckv_xxx ()
{
    statements-go-here
}


# Finally, set the global variable used to signify this library has been loaded
#
CKVAL_LIB_IS_LOADED=true
