#!/bin/ksh
#
# Copyright 2003 Sun Microsystems, Inc. All Rights Reserved.
#
# @(#)watch_lib.ksh	1.4 03/10/27
#
# watch_lib.ksh -- Library for ESM process "watch" management
#
# This library is intended to be imported by a Korn shell script
# operating in the ESM common scripting environment.
#
# NOTE: the esm_lib.ksh library is expected to be already loaded.
#
# Public API Quick Summary:
#
#   el_watch_init   - Initializes an watch_lib session
#   el_watch_add    - Add a watch entry to the pending "add" list
#   el_watch_remove - Add a watch entry to the pending "remove" list
#   el_watch_apply  - Apply "add" & "remove" entries for this session
#   el_watch_fini   - Terminates an watch_lib session
#
# Unless otherwise noted, all functions return 0 for success,
# or 1 if an error was encountered.  Error/warning messages are logged
# using the standard el_log and el_slog functions.
#
# This library is designed such that a single top-level script (that invokes
# subordinate scripts) uses the session-oriented functions to properly
# control the target watch entries to be added or removed.
#
# Any script may invoke el_watch_add and/or el_watch_remove, but a session
# must first be established via el_watch_init.  Once all watch entries have
# been added or removed, the final watch entry registration lists (for both
# add and remove) are "applied" via el_watch_apply.  Finally, el_watch_fini
# is invoked to terminate the session (cleanup, etc).
#
# All public functions defined herein have a "el_watch_" name prefix.
# All public variables have a "EW_" name prefix.

# All private (local) functions have a "_elw_" name prefix.
# All private (local) variables have a "__elw_" name prefix.
#
###

# Public variables
#
# Special hidden variable used to control unit testing
: ${EW_ENABLE_UNIT_TESTING:=/usr/bin/false}

# Default watch interval, in minutes
: ${EW_INTERVAL=5}

# Directory for storing temporary watch files
: ${ESM_TMP:=/tmp}
[ -d $ESM_TMP ] ||
    el_assert "Temporary directory for watch_lib does not exist: $ESM_TMP"

# Standard program signature used for crontab entries
#
if [ -z "$EW_WATCH_PROGRAM" ] ; then
    [ -n "$ESM_BIN" ] ||
	el_assert "ESM_BIN is not defined for watch_lib library"
    EW_WATCH_PROGRAM="$ESM_BIN/esmwatch"
    export EW_WATCH_PROGRAM
fi

if $EW_ENABLE_UNIT_TESTING ; then
    [ -x "$EW_WATCH_PROGRAM" ] ||
	el_assert "Standard watch program '$EW_WATCH_PROGRAM' not found or not executable"
fi

# Private variables
#
# File used for "add" entries
__elw_addfile=$ESM_TMP/watch_lib_add.entries

# File used for "remove" entries
__elw_remfile=$ESM_TMP/watch_lib_remove.entries

# Files used for constructing a crontab
__elw_cttmp=$ESM_TMP/esm_crontab.$$
__elw_cttmp_new=$ESM_TMP/esm_crontab_new.$$


# Scratch (temp) file used for whatever
__elw_tmpfile=$ESM_TMP/watch_lib.$$


# --------------------------------------------------------------------
# el_watch_add component_name [watch_interval [crontab_entry]]
#
# Add a watch entry for the named component.  You can optionally specify
# the watch interval (in minutes, default = $EW_INTERVAL).  You can
# also optionally specify the full, detailed crontab entry for this
# watch entry, but read the caveats below...
#
# If the crontab_entry is not specified (the typical case), then a
# standard crontab entry will be created (in el_watch_apply) that uses
# the standard "esmwatch" script with the resolved watch_interval.
#
# If you specify the crontab_entry, then when removing this entry,
# be sure to use the optional form of the el_watch_remove where
# you specify the the grep_pattern for identifying the crontab entry.
#
# Also note that a dummy placeholder watch_interval parameter MUST be
# specified when you specify the full crontab_entry parameter.  Just
# use something simple, like an "x" character, for example:
#
#    el_watch_add sssm x "<full crontab entry line>"
#
# Also be sure to wrap the crontab_entry in SINGLE-QUOTES to avoid
# nasty shell interpretation of any "*" characters in the entry!
#
el_watch_add () {

    __elw_compname=$1

    el_slog 1 "Adding entry to add the watch for '$__elw_compname'..."

    # Resolve the target interval
    #
    if [ -n "$2" ] ; then
        __elw_interval=$2
    else
        __elw_interval=$EW_INTERVAL
    fi

    # Resolve the optional crontab entry
    #
    if [ -n "$3" ] ; then
        __elw_cronentry=$3
    else
        __elw_cronentry=''
    fi

    # If the target component is already in the list, then remove
    # that entry, then append this entry to the list.
    #
    if [ -f $__elw_addfile ] && /usr/bin/egrep -s "^$__elw_compname" $__elw_addfile
    then
        if /usr/bin/egrep -v "^$__elw_compname " $__elw_addfile > $__elw_tmpfile &&
           /usr/bin/mv $__elw_tmpfile $__elw_addfile
        then
            :  # Success, old entry has been removed
        else
            el_log 1 "Failure to remove existing watch entry for '$__elw_compname' in $__elw_addfile"
            return 1
        fi
    fi

    print "$__elw_compname $__elw_interval $__elw_cronentry" >> $__elw_addfile

    # Return final exit status
}


# --------------------------------------------------------------------
# el_watch_apply
#
# Apply all "adds" and "removes" for the current watch_lib session.
#
el_watch_apply () {

    # Immediately return (with success) if no entries are present
    # in either list.
    #
    el_slog 1 "Applying watch entries..."

    if [ ! -s $__elw_remfile ] && [ ! -s $__elw_addfile ] ; then
        el_slog 1 "[No watch entries to add or remove at this time]"
        return 0
    fi

    # Default return code (set to 1 upon error)
    #
    __elw_rc=0

    # Fetch the current crontab list for editing
    #
    /usr/bin/crontab -l > $__elw_cttmp 2> $__elw_tmpfile
    if [ $? -ne 0 ] ; then
	el_slog 1 "`/usr/bin/cat $__elw_tmpfile`"
	#jw-- This fails when run in different locales
	#jw if /usr/bin/egrep -s "can't open your crontab file" $__elw_tmpfile
        #jw then
	#jw     el_slog 1 "[Current crontab file appears to be empty]"
	#jw else
        #jw     el_assert "Failure to acquire current crontab entries"
	#jw fi
    fi

    if $EW_ENABLE_UNIT_TESTING ; then
        echo "Initial crontab entry prior to editing:"
        echo " -----"
        /usr/bin/cat $__elw_cttmp
        echo " -----"
    fi

    # Delete all crontab entries targeted for removal
    #
    el_slog 1 "Removing watch entries from crontab..."

    if [ ! -s $__elw_remfile ] ; then
        el_slog 1 "[No watch entries to remove at this time]"
    else
        # Loop thru the remove list and construct the crontab entry
        # based on the list item contents, then search for and remove
        # that item in the crontab.
        #
        while read __elw_line
        do
            __elw_comp=`echo "$__elw_line" | /usr/bin/cut -d' ' -f1`
            __elw_ctpattern=`_elw_get_grep_pattern "$__elw_line"`

            if /usr/bin/grep "$__elw_ctpattern" $__elw_cttmp > /dev/null 2>&1
            then
		# Some pretty funky handling must be done here to remove
		# the matched entry in the temp crontab file due to the way
		# that 'egrep -v' works.  If this (matched) entry is the ONLY
		# line in the file, then an 'egrep -v' on the file will FAIL,
		# since there are no NON-MATCHING lines in the file (wierd...).
		# So, what we do is first check if the temp crontab file has
		# only a single line left, and if so, then nullify the file,
		# resulting in an empty crontab file (since we are trying to
		# remove the sole entry).
		#
		if /usr/bin/egrep -v "$__elw_ctpattern" $__elw_cttmp > $__elw_cttmp_new &&
		   /usr/bin/mv $__elw_cttmp_new $__elw_cttmp
		then
		    :
		else
		    # Check for and handle the single-line case
		    #
		    if [ `/usr/bin/wc -l $__elw_cttmp | /usr/bin/awk '{print $1}'` ]
		    then
			/usr/bin/cat /dev/null > $__elw_cttmp
		    else
			el_slog 1 "[WARNING: Failure to remove component '$__elw_comp' crontab entry]"
			__elw_rc=1
		    fi
		fi
            else
                el_slog 1 "[WARNING: Component '$__elw_comp' not found in crontab]"
            fi
        done < $__elw_remfile

	# All "remove" entries have been processed, so be sure to
	# delete the remove list file.
	#
	/usr/bin/rm $__elw_remfile ||
	    el_log 1 "WARNING: Failure to delete the watch 'remove' list file ($__elw_remfile)"
    fi

    # Create all crontab entries targeted for addition
    #
    el_slog 1 "Adding watch entries to crontab..."

    if [ ! -s $__elw_addfile ] ; then
        el_slog 1 "[No watch entries to add at this time]"
    else
        # Loop thru the remove list and construct the crontab entry
        # based on the list item contents, then add that entry to
        # the crontab contruction file.
        #
        while read __elw_line
        do
            __elw_comp=`echo "$__elw_line" | /usr/bin/cut -d' ' -f1`
            __elw_ctentry=`_elw_mk_ctentry "$__elw_line"`

            if echo "$__elw_ctentry" >> $__elw_cttmp ; then
                :
            else
                el_slog 1 "[WARNING: Failure to add component '$__elw_comp' crontab entry]"
                __elw_rc=1
            fi
        done < $__elw_addfile

	# All "add" entries have been processed, so be sure to
	# delete the add list file.
	#
	/usr/bin/rm $__elw_addfile ||
	    el_log 1 "WARNING: Failure to delete the watch 'add' list file ($__elw_addfile)"
    fi

    # Update the crontab
    #
    # If the crontab construction file is empty, then it implies
    # that we have just removed all registered cron entries, so
    # a different crontab command must be used in that case.
    #
    if $EW_ENABLE_UNIT_TESTING ; then
        echo "Final crontab entry:"
        echo " -----"
        /usr/bin/cat $__elw_cttmp
        echo " -----"
    fi

    if [ -s $__elw_cttmp ] ; then
	/usr/bin/crontab $__elw_cttmp
    else
	/usr/bin/crontab -r
    fi
    if [ $? -ne 0 ] ; then
        el_slog 1 "WARNING: Failure to update the final crontab"
        __elw_rc=1
    fi

    return $__elw_rc
}


# --------------------------------------------------------------------
# el_watch_fini
#
# Terminate (cleanup) an watch_lib session.
#
el_watch_fini () {

    el_slog 1 "Terminating watch_lib session"

    /usr/bin/rm -f $__elw_addfile &&
    /usr/bin/rm -f $__elw_remfile &&
    /usr/bin/rm -f $__elw_tmpfile &&
    return 0

    el_log 1 "Failure to terminate the watch_lib session"
    return 1
}


# --------------------------------------------------------------------
# el_watch_init
#
# Initialize (start) an watch_lib session.
#
el_watch_init () {

    el_slog 1 "Initializing the watch_lib session"

    /usr/bin/cat /dev/null > $__elw_addfile &&
    /usr/bin/cat /dev/null > $__elw_remfile &&
    return 0

    el_log 1 "Failure to initialize the watch_lib session"
    return 1
}


# --------------------------------------------------------------------
# el_watch_remove component_name [grep_pattern]
#
# Remove the watching of the named component.  If you had added the
# component using a fully-specified crontab_entry, then you must also
# specify the grep_pattern used to identify the entry in the crontab;
# in this case it is suggested that you specify the path or name of
# the script/program used in the crontab entry (but keep it simple!).
#
el_watch_remove () {

    __elw_compname=$1

    el_slog 1 "Adding entry to remove the watch for '$__elw_compname'..."

    # Resolve the optional grep_pattern
    #
    if [ -n "$2" ] ; then
        __elw_grep_pattern=$2
    else
        __elw_grep_pattern=''
    fi

    # If the target component is already in the list, then remove
    # that entry, then append this entry to the list.
    #
    if [ -f $__elw_remfile ] && /usr/bin/egrep -s "^$__elw_compname" $__elw_remfile
    then
        if /usr/bin/egrep -v "^$__elw_compname " $__elw_remfile > $__elw_tmpfile &&
           /usr/bin/mv $__elw_tmpfile $__elw_remfile
        then
            :  # Success, old entry has been removed
        else
            el_log 1 "Failure to remove existing watch entry for '$__elw_compname' in $__elw_remfile"
            return 1
        fi
    fi

    echo $__elw_compname $__elw_grep_pattern >> $__elw_remfile

    # Return final exit status

    return 0
}


# --------------------------------------------------------------------
# _elw_get_grep_pattern remove_list_entry
#
# Output the resolved grep pattern for the specified "remove" entry.
#
# NOTE: This function is (obviously) quite closely tied to the algorithm
# used in the _elw_mk_ctentry function.
#
_elw_get_grep_pattern () {

    __elw_pattern=`echo "$1" | /usr/bin/cut -d' ' -f2-`

    if [ -z "$__elw_pattern" ] ; then
        __elw_comp=`echo $1 | /usr/bin/cut -d' ' -f1`
        __elw_pattern="$EW_WATCH_PROGRAM $__elw_comp"
    fi

    echo "$__elw_pattern"
}


# --------------------------------------------------------------------
# _elw_mk_ctentry add_list_entry
#
# Output the resolved grep pattern for the specified "add" entry.
#
_elw_mk_ctentry () {

    __elw_ctentry=`echo "$1" | /usr/bin/cut -d' ' -f3-`

    if [ -z "$__elw_ctentry" ] ; then
        __elw_comp=`echo "$1" | /usr/bin/cut -d' ' -f1`
        __elw_interval=`echo "$1" | /usr/bin/cut -d' ' -f2`

	# Construct the interval sequence
	#
	__xx=$__elw_interval
	__seq=$__elw_interval

	while true
	do
	    __xx=`/usr/bin/expr $__xx + $__elw_interval`
	    [ $__xx -gt 60 ] && break

	    if [ $__xx -eq 60 ] ; then
		__seq="0,${__seq}"
	    else
		__seq="${__seq},$__xx"
	    fi
	done    

	__elw_ctentry="$__seq * * * * [ -x $EW_WATCH_PROGRAM ] && $EW_WATCH_PROGRAM $__elw_comp"

    fi

    echo "$__elw_ctentry"
}
